﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Infragistics.Web.Mvc;


namespace IgUploadMvc03
{
    [Serializable]
    public class CustomSafeDictionary : ISafeDictionary<string, UploadInfo>
    {

        #region Members
        private readonly object syncRoot = new object();
        private Dictionary<string, UploadInfo> _dictionary;
        #endregion


        public CustomSafeDictionary()
        {
            this._dictionary = new Dictionary<string, UploadInfo>();
            Serialize(this._dictionary);
        }

        #region Serialization
        private Dictionary<string, UploadInfo> Dictionary
        {
            get
            {
                return this.Deserialize();
            }
        }

        private Dictionary<string, UploadInfo> Deserialize()
        {
            UploadService.UploadServiceClient client = new UploadService.UploadServiceClient();
            return client.GetData();
        }

        private void Serialize(Dictionary<string, UploadInfo> d)
        {
            UploadService.UploadServiceClient client = new UploadService.UploadServiceClient();
            client.SetData(d);
        }
        #endregion

        #region Properties
        #region Keys
        /// <summary>
        /// Returns the collection of keys
        /// </summary>
        public ICollection<string> Keys
        {
            get
            {
                ICollection<string> keys;
                lock (syncRoot)
                {
                    keys = this.Dictionary.Keys;
                }
                return keys;
            }
        }
        #endregion

        #region Values
        /// <summary>
        /// Gets the collection containing the values
        /// </summary>
        public ICollection<UploadInfo> Values
        {
            get
            {
                ICollection<UploadInfo> values;
                lock (syncRoot)
                {
                    values = this.Dictionary.Values;
                }
                return values;
            }
        }
        #endregion

        #region this[string key]
        /// <summary>
        /// Gets or sets the value associated with the specified key.
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public UploadInfo this[string key]
        {
            get
            {
                UploadInfo value;
                lock (syncRoot)
                {
                    value = this.Dictionary[key];
                }
                return value;
            }
            set
            {
                lock (syncRoot)
                {
                    this._dictionary = this.Dictionary;
                    this._dictionary[key] = value;
                    this.Serialize(this._dictionary);

                }
            }
        }
        #endregion

        #region Count
        /// <summary>
        /// Gets the number of key/value pairs in the dictionary
        /// </summary>
        public int Count
        {
            get
            {
                int count = 0;
                lock (syncRoot)
                {
                    count = this.Dictionary.Count;
                }
                return count;
            }
        }
        #endregion

        #region IsReadOnly
        /// <summary>
        /// Returns whether the dictionary is read only
        /// </summary>
        public bool IsReadOnly
        {
            get { return false; }
        }
        #endregion
        #endregion

        #region Methods
        #region IDictionary<string,UploadInfoMembers>
        /// <summary>
        /// Adds the specified key and value to the dictionary
        /// </summary>
        /// <param name="key"></param>
        /// <param name="value"></param>
        public void Add(string key, UploadInfo value)
        {
            lock (syncRoot)
            {
                this._dictionary = this.Dictionary;
                this._dictionary.Add(key, value);
                this.Serialize(this._dictionary);
            }
        }

        /// <summary>
        /// Check if the key exists in the dictionary, if not adds the value
        /// </summary>
        /// <param name="key"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public bool CheckAndAdd(string key, UploadInfo value)
        {
            bool isAdded = false;
            lock (syncRoot)
            {
                if (!this.Dictionary.ContainsKey(key))
                {
                    this._dictionary = this.Dictionary;
                    this._dictionary.Add(key, value);
                    this.Serialize(this._dictionary);
                    isAdded = true;
                }
            }

            return isAdded;
        }

        /// <summary>
        /// Check if the key exists in dictionary, if not adds the value
        /// </summary>
        /// <param name="keyValuePair"></param>
        /// <returns></returns>
        public bool CheckAndAdd(KeyValuePair<string, UploadInfo> keyValuePair)
        {
            bool isAdded = false;
            lock (syncRoot)
            {
                if (!this.Dictionary.ContainsKey(keyValuePair.Key))
                {
                    Add(keyValuePair);
                    isAdded = true;
                }
            }

            return isAdded;
        }

        /// <summary>
        /// Check if the dictionary contains the specified key
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public bool ContainsKey(string key)
        {
            bool isContains = false;
            lock (syncRoot)
            {
                isContains = this.Dictionary.ContainsKey(key);
            }
            return isContains;
        }


        /// <summary>
        /// Removes from the dictionary item with the specified key
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public bool Remove(string key)
        {
            lock (syncRoot)
            {
                UploadService.UploadServiceClient client = new UploadService.UploadServiceClient();

                return client.RemoveData(key);
            }
        }

        public bool TryGetValue(string key, out UploadInfo value)
        {
            lock (syncRoot)
            {
                this._dictionary = this.Dictionary;
                return this._dictionary.TryGetValue(key, out value);
            }
        }


        #endregion

        #region ICollection<KeyValuePair<string,UploadInfo>Members
        /// <summary>
        /// Adds the item to collection
        /// </summary>
        /// <param name="item"></param>
        public void Add(KeyValuePair<string, UploadInfo> item)
        {
            lock (syncRoot)
            {
                this._dictionary = this.Dictionary;
                ((ICollection<KeyValuePair<string, UploadInfo>>)this._dictionary).Add(item);
                this.Serialize(this._dictionary);
            }
        }

        /// <summary>
        /// Removes all keys and values from the dictionary
        /// </summary>
        public void Clear()
        {
            lock (syncRoot)
            {
                this._dictionary = this.Dictionary;
                this._dictionary.Clear();
                this.Serialize(this._dictionary);
            }
        }

        /// <summary>
        /// Check whether the dictionary contains the specific item
        /// </summary>
        /// <param name="item"></param>
        /// <returns></returns>
        public bool Contains(KeyValuePair<string, UploadInfo> item)
        {
            return ((ICollection<KeyValuePair<string,
            UploadInfo>>)this.Dictionary).Contains(item);
        }

        /// <summary>
        /// Copies the dictionary KeyCollection
        /// elements to an existing one-dimensional System.Array, starting at the specified array index.
        /// </summary>
        /// <param name="array"></param>
        /// <param name="arrayIndex"></param>
        public void CopyTo(KeyValuePair<string, UploadInfo>[] array, int arrayIndex)
        {
            lock (syncRoot)
            {
                this._dictionary = this.Dictionary;
                ((ICollection<KeyValuePair<string, UploadInfo>>)this._dictionary).CopyTo(array,
                arrayIndex);
                this.Serialize(this._dictionary);
            }
        }

        /// <summary>
        /// Removes the value with the specified key from the dictionary.
        /// </summary>
        /// <param name="item"></param>
        /// <returns></returns>
        public bool Remove(KeyValuePair<string, UploadInfo> item)
        {
            lock (syncRoot)
            {
                this._dictionary = this.Dictionary;
                return ((ICollection<KeyValuePair<string,
                UploadInfo>>)this._dictionary).Remove(item);
                this.Serialize(this._dictionary);
            }
        }

        #endregion

        #region IEnumerable<KeyValuePair<string,UploadInfo>Members
        /// <summary>
        /// Returns an enumerator that iterates through the collection.
        /// </summary>
        /// <returns></returns>
        public IEnumerator<KeyValuePair<string, UploadInfo>> GetEnumerator()
        {
            return ((ICollection<KeyValuePair<string, UploadInfo>>)this.Dictionary).GetEnumerator();
        }

        #endregion

        #region IEnumerable Members
        /// <summary>
        /// Returns an enumerator that iterates through a collection.
        /// </summary>
        /// <returns>An System.Collections.IEnumerator object that can be used to iterate through the collection</returns>
        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return ((System.Collections.IEnumerable)this.Dictionary).GetEnumerator();
        }

        #endregion
        #endregion
    }
}